@@ -11,6 +11,20 @@ import cairo
 import pytest
 
 
+def test_tee_surface():
+    main = cairo.ImageSurface(cairo.FORMAT_ARGB32, 10, 10)
+    tee = cairo.TeeSurface(main)
+    assert isinstance(tee, cairo.TeeSurface)
+
+    # the API is horrible, passing a wrong arg sets the surface to an error
+    # state instead of returning the status.
+    s1 = cairo.ImageSurface(cairo.FORMAT_ARGB32, 10, 10)
+    tee.add(s1)
+    assert tee.index(0)
+    assert tee.index(1)
+    tee.remove(s1)
+
+
 def test_image_surface_get_data_refcount():
     surface = cairo.ImageSurface(cairo.FORMAT_ARGB32, 10, 10)
     assert sys.getrefcount(surface) == 2
@@ -20,6 +34,14 @@ def test_image_surface_get_data_refcount():
     assert sys.getrefcount(surface) == 2
 
 
+def test_image_surface_get_data_crasher():
+    surface = cairo.ImageSurface(cairo.FORMAT_ARGB32, 10, 10)
+    try:
+        assert 0, surface.get_data()
+    except Exception:
+        pass
+
+
 def test_surface_get_content():
     surface = cairo.ImageSurface(cairo.FORMAT_ARGB32, 10, 10)
     assert surface.get_content() == cairo.Content.COLOR_ALPHA
@@ -50,6 +72,22 @@ def test_svg_surface_get_versions():
     assert all(isinstance(v, cairo.SVGVersion) for v in versions)
 
 
+def test_surface_get_device_scale():
+    surface = cairo.ImageSurface(cairo.FORMAT_ARGB32, 10, 10)
+    device_scale = surface.get_device_scale()
+    assert all(isinstance(s, float) for s in device_scale)
+
+
+def test_surface_set_device_scale():
+    surface = cairo.ImageSurface(cairo.FORMAT_ARGB32, 10, 10)
+    ret = surface.set_device_scale(5.0, 3.0)
+    assert ret is None
+
+    surface = cairo.ImageSurface(cairo.FORMAT_ARGB32, 100, 100)
+    with pytest.raises(cairo.Error):
+        surface.set_device_scale(1, 0)
+
+
 @pytest.mark.skipif(not cairo.HAS_PNG_FUNCTIONS, reason="not png support")
 def test_image_surface_create_for_data_array():
     width, height = 255, 255
@@ -129,3 +167,37 @@ def test_surface_from_stream_closed_before_finished():
         fileobj.close()
         with pytest.raises(IOError):
             surface.finish()
+
+
+def test_script_surface():
+    f = io.BytesIO()
+    dev = cairo.ScriptDevice(f)
+    surface = cairo.ScriptSurface(dev, cairo.Content.COLOR_ALPHA, 42, 10)
+    assert isinstance(surface, cairo.ScriptSurface)
+    cairo.Context(surface).paint()
+    dev.flush()
+    assert b"42" in f.getvalue()
+    assert b"paint" in f.getvalue()
+
+
+def test_script_surface_create_for_target():
+    # paint the script proxy
+    f = io.BytesIO()
+    dev = cairo.ScriptDevice(f)
+    surface = cairo.ImageSurface(cairo.FORMAT_ARGB32, 10, 10)
+    script = cairo.ScriptSurface.create_for_target(dev, surface)
+    assert isinstance(script, cairo.ScriptSurface)
+    ctx = cairo.Context(script)
+    ctx.set_source_rgb(0.25, 0.5, 1.0)
+    ctx.paint()
+    assert b"paint" in f.getvalue()
+    surface.flush()
+    image_data = bytes(surface.get_data())
+
+    # check if the result is the same
+    surface = cairo.ImageSurface(cairo.FORMAT_ARGB32, 10, 10)
+    ctx = cairo.Context(surface)
+    ctx.set_source_rgb(0.25, 0.5, 1.0)
+    ctx.paint()
+    surface.flush()
+    assert bytes(surface.get_data()) == image_data
